##################### generated by xml-casa (v2) from hifa_importdata.xml ###########
##################### c918931004871bc6a0a5510c8d0b3967 ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor

_pc = _val_ctor()
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_hifa_importdata import hifa_importdata as _hifa_importdata_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log


class _hifa_importdata:
    """
    hifa_importdata ---- Imports data into the interferometry pipeline


    The hifa_importdata task loads the specified visibility data into the pipeline
    context unpacking and / or converting it as necessary.

    If the 'overwrite' input parameter is set to False and the task is
    asked to convert an input ASDM input to an MS, then when the output
    MS already exists in the output directory, the importasdm
    conversion step is skipped, and the existing MS will be imported
    instead.

    results -- If pipeline mode is 'getinputs' then None is returned. Otherwise
    the results object for the pipeline task is returned.


    --------- parameter descriptions ---------------------------------------------

    vis               List of visibility data files.
                      These may be ASDMs, tar files of ASDMs,
                      MSes, or tar files of MSes. If ASDM files are specified, they will be
                      converted to MS format.
                      example: vis=['X227.ms', 'asdms.tar.gz']
    session           List of visibility data sessions
    pipelinemode      The pipeline operating mode.
                      In 'automatic' mode the pipeline determines the values of all
                      context defined pipeline inputs automatically.
                      In 'interactive' mode the user can set the pipeline context defined
                      parameters manually. In 'getinputs' mode the user can check the settings of
                      all pipeline parameters without running the task.
    asis              Extra ASDM tables to convert as is
    process_caldevice Import the caldevice table from the ASDM
    overwrite         Overwrite existing files on import.
                      Can only be set in pipelinemode='interactive'.
                      When converting ASDM to MS, if overwrite=False and the MS
                      already exists in output directory, then this existing MS
                      dataset will be used instead.
    nocopy            Disable copying of MS to working directory
    bdfflags          Apply BDF flags on import
    datacolumns       Dictionary defining the data types of
                      existing columns. The format is:

                      {'data': 'data type 1'}
                      or
                      {'data': 'data type 1', 'corrected': 'data type 2'}

                      For ASDMs the data type can only be RAW and one
                      can only specify it for the data column.

                      For MSes one can define two different data types
                      for the DATA and CORRECTED_DATA columns and they
                      can be any of the known data types (RAW,
                      REGCAL_CONTLINE_ALL, REGCAL_CONTLINE_SCIENCE,
                      SELFCAL_CONTLINE_SCIENCE, REGCAL_LINE_SCIENCE,
                      SELFCAL_LINE_SCIENCE, BASELINED, ATMCORR). The
                      intent selection strings _ALL or _SCIENCE can be
                      skipped. In that case the task determines this
                      automatically by inspecting the existing intents
                      in the dataset.

                      Usually, a single datacolumns dictionary is used
                      for all datasets. If necessary, one can define a
                      list of dictionaries, one for each EB, with
                      different setups per EB.

                      If no types are specified,
                      {'data':'raw','corrected':'regcal_contline'}
                      or {'data':'raw'} will be assumed, depending on
                      whether the corrected column exists or not.
    lazy              Use the lazy filler import
    dbservice         Use the online flux catalog
    ocorr_mode        ALMA default set to ca
    createmms         Create an MMS
    minparang         Minimum required parallactic angle range for polarisation calibrator,
                      in degrees. The default of 0.0 is used for non-polarisation processing.
    dryrun            Run the task (False) or display task command (True)
    acceptresults     Add the results into the pipeline context

    --------- examples -----------------------------------------------------------


    1. Load an ASDM list in the ../rawdata subdirectory into the context:

    hifa_importdata(vis=['../rawdata/uid___A002_X30a93d_X43e',
    '../rawdata/uid_A002_x30a93d_X44e'])

    2. Load an MS in the current directory into the context:

    hifa_importdata(vis=['uid___A002_X30a93d_X43e.ms'])

    3. Load a tarred ASDM in ../rawdata into the context:

    hifa_importdata(vis=['../rawdata/uid___A002_X30a93d_X43e.tar.gz'])

    4. Check the hif_importdata inputs, then import the data:

    myvislist = ['uid___A002_X30a93d_X43e.ms', 'uid_A002_x30a93d_X44e.ms']
    hifa_importdata(vis=myvislist, pipelinemode='getinputs')
    hifa_importdata(vis=myvislist)

    5. Load an ASDM but check the results before accepting them into the context.

    results = hifa_importdata(vis=['uid___A002_X30a93d_X43e.ms'],
    acceptresults=False)
    results.accept()

    6. Run in dryrun mode before running for real:

    results = hifa_importdata(vis=['uid___A002_X30a93d_X43e.ms'], dryrun=True)
    results = hifa_importdata(vis=['uid___A002_X30a93d_X43e.ms'])

    7. Run with explicit setting of data column types:

    hifa_importdata(vis=['uid___A002_X30a93d_X43e_targets.ms'], datacolumns={'data': 'regcal_contline'})
    hifa_importdata(vis=['uid___A002_X30a93d_X43e_targets_line.ms'], datacolumns={'data': 'regcal_line', 'corrected': 'selfcal_line'})



    """

    _info_group_ = """pipeline"""
    _info_desc_ = """Imports data into the interferometry pipeline"""

    def __call__(
        self,
        vis=[],
        session=[],
        pipelinemode="automatic",
        asis="SBSummary ExecBlock Annotation Antenna Station Receiver Source CalAtmosphere CalWVR CalPointing",
        process_caldevice=False,
        overwrite=False,
        nocopy=False,
        bdfflags=True,
        datacolumns=[],
        lazy=False,
        dbservice=False,
        ocorr_mode="ca",
        createmms="false",
        minparang=float(0.0),
        dryrun=False,
        acceptresults=True,
    ):
        schema = {
            "vis": {
                "type": "cStrVec",
                "coerce": [_coerce.to_list, _coerce.to_strvec],
            },
            "session": {
                "type": "cStrVec",
                "coerce": [_coerce.to_list, _coerce.to_strvec],
            },
            "pipelinemode": {
                "type": "cStr",
                "coerce": _coerce.to_str,
                "allowed": ["automatic", "interactive", "getinputs"],
            },
            "asis": {"type": "cStr", "coerce": _coerce.to_str},
            "process_caldevice": {"type": "cBool"},
            "overwrite": {"type": "cBool"},
            "nocopy": {"type": "cBool"},
            "bdfflags": {"type": "cBool"},
            "datacolumns": {
                "type": "cVariant",
                "coerce": [_coerce.to_variant],
            },
            "lazy": {"type": "cBool"},
            "dbservice": {"type": "cBool"},
            "ocorr_mode": {"type": "cStr", "coerce": _coerce.to_str},
            "createmms": {
                "type": "cStr",
                "coerce": _coerce.to_str,
                "allowed": ["false", "true", "automatic"],
            },
            "minparang": {"type": "cFloat", "coerce": _coerce.to_float},
            "dryrun": {"type": "cBool"},
            "acceptresults": {"type": "cBool"},
        }
        doc = {
            "vis": vis,
            "session": session,
            "pipelinemode": pipelinemode,
            "asis": asis,
            "process_caldevice": process_caldevice,
            "overwrite": overwrite,
            "nocopy": nocopy,
            "bdfflags": bdfflags,
            "datacolumns": datacolumns,
            "lazy": lazy,
            "dbservice": dbservice,
            "ocorr_mode": ocorr_mode,
            "createmms": createmms,
            "minparang": minparang,
            "dryrun": dryrun,
            "acceptresults": acceptresults,
        }
        assert _pc.validate(doc, schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log(
            "hifa_importdata",
            [
                "vis=" + repr(_pc.document["vis"]),
                "session=" + repr(_pc.document["session"]),
                "pipelinemode=" + repr(_pc.document["pipelinemode"]),
                "asis=" + repr(_pc.document["asis"]),
                "process_caldevice=" + repr(_pc.document["process_caldevice"]),
                "overwrite=" + repr(_pc.document["overwrite"]),
                "nocopy=" + repr(_pc.document["nocopy"]),
                "bdfflags=" + repr(_pc.document["bdfflags"]),
                "datacolumns=" + repr(_pc.document["datacolumns"]),
                "lazy=" + repr(_pc.document["lazy"]),
                "dbservice=" + repr(_pc.document["dbservice"]),
                "ocorr_mode=" + repr(_pc.document["ocorr_mode"]),
                "createmms=" + repr(_pc.document["createmms"]),
                "minparang=" + repr(_pc.document["minparang"]),
                "dryrun=" + repr(_pc.document["dryrun"]),
                "acceptresults=" + repr(_pc.document["acceptresults"]),
            ],
        )
        task_result = None
        try:
            task_result = _hifa_importdata_t(
                _pc.document["vis"],
                _pc.document["session"],
                _pc.document["pipelinemode"],
                _pc.document["asis"],
                _pc.document["process_caldevice"],
                _pc.document["overwrite"],
                _pc.document["nocopy"],
                _pc.document["bdfflags"],
                _pc.document["datacolumns"],
                _pc.document["lazy"],
                _pc.document["dbservice"],
                _pc.document["ocorr_mode"],
                _pc.document["createmms"],
                _pc.document["minparang"],
                _pc.document["dryrun"],
                _pc.document["acceptresults"],
            )
        except Exception as exc:
            _except_log("hifa_importdata", exc)
            raise
        finally:
            task_result = _end_log(
                _logging_state_, "hifa_importdata", task_result
            )
        return task_result


hifa_importdata = _hifa_importdata()
